using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using System.IO;
//using System;
//using UnityEditor.AssetImporters;
using Random = UnityEngine.Random;


/// <summary>
/// NOTE: I commented out all "Material" related stuff because, dispite a lot of research, I could not find a way to simply make a specified material, the default material for an 
/// imported model. I found lots of ways to do searches for the materials in the asset folders and things, but no way to simply assign the materials directly.
/// </summary>

[ExecuteInEditMode]

public class ModelImportFormatter : EditorWindow
{
	#region General
	private string importDirPath = "";
	private bool isValidFolderSelection = false;
	private Vector2 scrollPosition;
	private ViewTab tab = ViewTab.Model;
	#endregion

	#region Model Tab
	private bool overwriteModelValues_Scene = false;
	private float scaleFactor = 1; 
	private bool convertUnits = true; 
	private bool bakeAxisConversion = false; 
	private bool importBlendShapes = true; 
	private bool importVisibility = true; 
	private bool importCameras = true; 
	private bool importLights = true; 
	private bool preserveHierarchy = false;
	private bool sortHeiarchyByName = true;

	private bool overwriteModelValues_Meshes = false;
	private ModelImporterMeshCompression meshCompression;
	private bool readWriteEnabled = false;
	private MeshOptimizationFlags optimizeMesh = MeshOptimizationFlags.Everything;
	private bool generateColliders = false;

	private bool overwriteModelValues_Geometry = false;
	private bool keepQuads = false;
	private bool weldVertices = true;
	private ModelImporterIndexFormat indexFormat;
	//private bool legacyBlendShapeNormals = false;
	private ModelImporterNormals normals;
	//private ModelImporterNormals blendShapeNormals;
	private ModelImporterNormalCalculationMode normalCalculationMode = ModelImporterNormalCalculationMode.AreaAndAngleWeighted;
	private ModelImporterNormalSmoothingSource normalSmoothingSource;
	[Range(0, 180)] private int normalSmoothingAngle;
	private ModelImporterTangents tangentSpaceMode = ModelImporterTangents.CalculateMikk;
	
	private bool swapUVs = false;
	private bool generateLightmapUVs = false;

	#endregion

	#region Rig Tab
	private bool overwriteRigValues = false;
	private ModelImporterAnimationType animType = ModelImporterAnimationType.Generic;
	//private ModelImporterAvatarSetup avatDef = ModelImporterAvatarSetup.NoAvatar;
	private ModelImporterSkinWeights skinWeights = ModelImporterSkinWeights.Standard;
	private int maxBonesPerVertex = 4;
	private float minBoneWeight = 0.001f;
	#endregion

	#region Anim Tab
	private bool overwriteAnimValues = false;
	private bool importConstraints = false;
	private bool importAnimations = true;
	#endregion

	//#region Mat Tab
	//private bool overwriteMatValues = false;
	//private MaterialMode materialMode = MaterialMode.DifferentMaterialsOnEach;
	//private MaterialMode lastMaterialMode = MaterialMode.DifferentMaterialsOnEach;
	//private Material masterMat; //this is the material that would be applied to all models if that option is selected
	//private bool lastToggleGroupState = true;
	//private bool toggleGroupState = true;
	//private Dictionary<string, Material> fileMatsDictionary;
	//private List<bool> perModelApplyMat; //this is the bool array that holds the options about whether or not to apply a material to each model.
	//private enum MaterialMode { SameMaterialOnAll, DifferentMaterialsOnEach, MasterMaterialCoversDisabledEntries};

	//#endregion

	private enum ViewTab { Model, Rig, Animation };//, Materials };

	#region Menus
	[MenuItem("Tools/Model Formatter")]
	public static void ShowWindow()
	{
		GetWindow<ModelImportFormatter>("Model Formatter");
	}

	[MenuItem("File/The Mystery Button//The Purple Button/The Blue Button/The Cyan Button/The Green Button/The Yellow Button/The Orange Button/The Infamous Red Button")]
	public static void Button()
	{
		Debug.LogWarning("You shouldn't have pressed that...  WARNING: PROJCT WILL SELF-DESTRUCT IN...   3...   2...   1...  ");
		Debug.LogError("( <<<<< BOOM >>>>> )");
		Debug.LogError("Your project is now forever ruined... 8(");
	}
	#endregion

	void Awake()
	{
		scrollPosition = new Vector2(0, 0);
		//fileMatsDictionary = new Dictionary<string, Material>();
		//perModelApplyMat = new List<bool>();
		//materialMode = MaterialMode.DifferentMaterialsOnEach;
	}
	private void OnGUI()
	{
		//if(fileMatsDictionary == null)
		//{
		//	fileMatsDictionary = new Dictionary<string, Material>();
		//}
		//Set label width
		float halfWindowWidth = EditorGUIUtility.currentViewWidth / 2;
		//Debug.Log("Half: " + halfWindowWidth + "   Label: " + EditorGUIUtility.labelWidth);
		EditorGUIUtility.labelWidth = halfWindowWidth;

		//Title
		EditorGUILayout.LabelField("3D Model Import Settings Mass Formatter", EditorStyles.boldLabel);

		//Start Scroll View
		scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition);

		//===== Set up Folder Selection =====
		string statedPath = "/>> NONE OR INVALID <<";
		if(string.IsNullOrEmpty(importDirPath) == false)
		{
			statedPath = importDirPath.Remove(0, Application.dataPath.Length);
		}

		EditorGUILayout.LabelField("Folder: /Assets" + statedPath);
		if(GUILayout.Button("Select Folder") == true)
		{
			importDirPath = EditorUtility.OpenFolderPanel("Select Folder with Models in it", Application.dataPath, "");
			if (importDirPath.StartsWith(Application.dataPath) == false)
			{
				importDirPath = "";
				isValidFolderSelection = false;
			}
			else
			{
				isValidFolderSelection = true;
			}
		}

		if(string.IsNullOrEmpty(importDirPath) == true || isValidFolderSelection == false)
		{
			EditorGUILayout.HelpBox("Please Select a valid folder with 3D Models in it", MessageType.Warning);
		}

		//===== Set up Model Import Settings =====
		EditorGUI.BeginDisabledGroup(string.IsNullOrEmpty(importDirPath) == true || isValidFolderSelection == false);
		EditorGUILayout.Space();
		DrawTabButtons();
		EditorGUILayout.Space();
		switch(tab)
		{
			case ViewTab.Rig:
				DrawRigSettings();
				break;
			case ViewTab.Animation:
				DrawAnimSettings();
				break;
			//case ViewTab.Materials:
			//	DrawMatSettings();
			//	break;
			default: //also ViewTab.Model
				DrawModelSettings();				
				break;
		}
		EditorGUI.EndDisabledGroup();

		if (GUILayout.Button("Apply All Settings", GUILayout.Height(30)) == true && importDirPath != "")
		{
			ImportModels();
		}

		EditorGUILayout.EndScrollView();
	}

	private void DrawTabButtons()
	{
		EditorGUILayout.BeginHorizontal();

		EditorGUILayout.Space(55);

		//Model Button
		EditorGUI.BeginDisabledGroup(tab == ViewTab.Model);
		if (GUILayout.Button("Model", GUILayout.Width(70), GUILayout.Height(20)))
		{
			tab = ViewTab.Model;
		}
		EditorGUI.EndDisabledGroup();

		//Rig Button
		EditorGUI.BeginDisabledGroup(tab == ViewTab.Rig);
		if (GUILayout.Button("Rig", GUILayout.Width(50), GUILayout.Height(20)))
		{
			tab = ViewTab.Rig;
		}
		EditorGUI.EndDisabledGroup();

		//Model Button
		EditorGUI.BeginDisabledGroup(tab == ViewTab.Animation);
		if (GUILayout.Button("Animation", GUILayout.Width(90), GUILayout.Height(20)))
		{
			tab = ViewTab.Animation;
		}
		EditorGUI.EndDisabledGroup();

		//Model Button
		/*EditorGUI.BeginDisabledGroup(tab == ViewTab.Materials);
		if (GUILayout.Button("Materials", GUILayout.Width(70), GUILayout.Height(20)))
		{
			tab = ViewTab.Materials;
		}
		EditorGUI.EndDisabledGroup();*/

		EditorGUILayout.Space(55);

		EditorGUILayout.EndHorizontal();
	}

	private void DrawModelSettings()
	{
		//Scene
		EditorGUILayout.LabelField("Scene", EditorStyles.boldLabel);
		overwriteModelValues_Scene = EditorGUILayout.Toggle("Overwrite Scene Values?", overwriteModelValues_Scene);
		EditorGUI.BeginDisabledGroup(overwriteModelValues_Scene == false);
		scaleFactor = EditorGUILayout.DelayedFloatField("Scale Factor", scaleFactor);
		convertUnits = EditorGUILayout.Toggle("Convert Units", convertUnits);
		bakeAxisConversion = EditorGUILayout.Toggle("Bake Axis Conversion", bakeAxisConversion);
		importBlendShapes = EditorGUILayout.Toggle("Import Blend Shapes", importBlendShapes);
		importVisibility = EditorGUILayout.Toggle("Import Visibility", importVisibility);
		importCameras = EditorGUILayout.Toggle("Import Cameras", importCameras);
		importLights = EditorGUILayout.Toggle("Import Lights", importLights);
		preserveHierarchy = EditorGUILayout.Toggle("Preserve Hierarchy", preserveHierarchy);
		sortHeiarchyByName = EditorGUILayout.Toggle("Sort Hierarchy By Name", sortHeiarchyByName);
		EditorGUI.EndDisabledGroup();
		EditorGUILayout.Space();

		//Meshes
		EditorGUILayout.LabelField("Meshes", EditorStyles.boldLabel);
		overwriteModelValues_Meshes = EditorGUILayout.Toggle("Overwrite Mesh Values?", overwriteModelValues_Meshes);
		EditorGUI.BeginDisabledGroup(overwriteModelValues_Meshes == false);
		meshCompression = (ModelImporterMeshCompression)EditorGUILayout.EnumPopup("Mesh Compression", meshCompression);
		readWriteEnabled = EditorGUILayout.Toggle("Read/Write Enabled", readWriteEnabled);
		optimizeMesh = (MeshOptimizationFlags) EditorGUILayout.EnumFlagsField("Optimize Mesh", optimizeMesh);
		generateColliders = EditorGUILayout.Toggle("Generate Colliders", generateColliders);
		EditorGUI.EndDisabledGroup();
		EditorGUILayout.Space();

		//Geometry
		EditorGUILayout.LabelField("Geometry", EditorStyles.boldLabel);
		overwriteModelValues_Geometry = EditorGUILayout.Toggle("Overwrite Geometry Values?", overwriteModelValues_Geometry);
		EditorGUI.BeginDisabledGroup(overwriteModelValues_Geometry == false);
		keepQuads = EditorGUILayout.Toggle("Keep Quads", keepQuads);
		weldVertices = EditorGUILayout.Toggle("Weld Vertices", weldVertices);
		indexFormat = (ModelImporterIndexFormat)EditorGUILayout.EnumPopup("Index Format", indexFormat);
		//legacyBlendShapeNormals = EditorGUILayout.Toggle("Legacy Blend Shape Normals", legacyBlendShapeNormals);
		normals = (ModelImporterNormals)EditorGUILayout.EnumPopup("Normals", normals);
		//blendShapeNormals = (ModelImporterNormals)EditorGUILayout.EnumPopup("Blend Shape Normals", blendShapeNormals);
		normalCalculationMode = (ModelImporterNormalCalculationMode)EditorGUILayout.EnumPopup("Normal Mode", normalCalculationMode);
		normalSmoothingSource = (ModelImporterNormalSmoothingSource)EditorGUILayout.EnumPopup("Smoothing Source", normalSmoothingSource);
		normalSmoothingAngle = EditorGUILayout.IntSlider("Smoothing Angle", normalSmoothingAngle, 0, 180);
		tangentSpaceMode = (ModelImporterTangents)EditorGUILayout.EnumPopup("Tangents", tangentSpaceMode);
		swapUVs = EditorGUILayout.Toggle("Swap UVs", swapUVs);
		generateLightmapUVs = EditorGUILayout.Toggle("Generate Lightmap UVs", generateLightmapUVs);
		
		EditorGUI.EndDisabledGroup(); //geometry
		EditorGUI.EndDisabledGroup(); //models tab
	}
	private void DrawRigSettings()
	{
		overwriteRigValues = EditorGUILayout.Toggle("Overwrite Values?", overwriteRigValues);
		EditorGUI.BeginDisabledGroup(overwriteRigValues == false);
		animType = (ModelImporterAnimationType)EditorGUILayout.EnumPopup("Animation Type", animType);
		EditorGUILayout.Space();
		//avatDef = (ModelImporterAvatarSetup)EditorGUILayout.EnumPopup("Avatar Definition", avatDef); This only works on one model at a time, so I didn't include it here.
		skinWeights = (ModelImporterSkinWeights)EditorGUILayout.EnumPopup("Skin Weights", skinWeights);
		if(skinWeights == ModelImporterSkinWeights.Custom)
		{
			maxBonesPerVertex = EditorGUILayout.IntSlider("Max Bones/Vertex",maxBonesPerVertex, 1, 255);
			minBoneWeight = EditorGUILayout.Slider("Min Bone Weight",minBoneWeight, 0.001f, 0.5f);
		}
		EditorGUI.EndDisabledGroup();
	}
	private void DrawAnimSettings()
	{
		overwriteAnimValues = EditorGUILayout.Toggle("Overwrite Values?", overwriteAnimValues);
		EditorGUI.BeginDisabledGroup(overwriteAnimValues == false);
		importConstraints = EditorGUILayout.Toggle("Import Constraints", importConstraints);
		importAnimations = EditorGUILayout.Toggle("Import Animations", importAnimations);
		EditorGUI.EndDisabledGroup();
	}
	/*private void DrawMatSettings()
	{
	//STILL NEED TO IMPLEMENT: Dropdown for switching between material modes, actually appling the materials during the import, field for master material, per model material toggles
		overwriteMatValues = EditorGUILayout.Toggle("Overwrite Values?", overwriteMatValues);
		EditorGUILayout.HelpBox("WARNING: Renaming a model will reset any unapplied settings in this tool upon refreshing the list!!\n-----\nChanging the selected folder above will reset any unapplied settings from the previous folder!!", MessageType.Warning);

		materialMode = (MaterialMode)EditorGUILayout.EnumPopup("Select Material Mode: ", materialMode);
		if (materialMode != MaterialMode.DifferentMaterialsOnEach)
		{
			masterMat = (Material)EditorGUILayout.ObjectField("Master Material: ", masterMat, typeof(Material), false);
		}
		EditorGUILayout.Space();
		if (GUILayout.Button("Refresh Model List") == true)
		{
			RefreshModelsList();
		}
		EditorGUI.BeginDisabledGroup(overwriteMatValues == false);
		if (fileMatsDictionary.Count > 0)
		{
			Dictionary<string, Material>.KeyCollection keysColl = fileMatsDictionary.Keys;
			string[] keys = new string[keysColl.Count];
			keysColl.CopyTo(keys, 0);
			if (lastMaterialMode != materialMode)
			{
				//Undo.RecordObject(this, "Change Material Mode to " + materialMode.ToString());
				if (materialMode == MaterialMode.SameMaterialOnAll)
				{
					for (int i = 0; i < perModelApplyMat.Count - 1; i++)
					{
						perModelApplyMat[i] = false;
					}
				}
				else if (materialMode == MaterialMode.MasterMaterialCoversDisabledEntries)
				{
					for (int i = 0; i < perModelApplyMat.Count - 1; i++)
					{
						if (perModelApplyMat[i] == false)
						{
							fileMatsDictionary[keys[i]] = masterMat;
						}
					}
				}
			}
			EditorGUILayout.Space();
			toggleGroupState = EditorGUILayout.BeginToggleGroup("Enable/Disable All?", toggleGroupState);
			for (int num = 0; num < keys.Length; num++)
			{
				string[] nameParts = keys[num].Split('/', '\\');
				string fileName = (string)nameParts.GetValue(nameParts.Length - 1);
				EditorGUILayout.BeginHorizontal();
				if(toggleGroupState != lastToggleGroupState)
				{ perModelApplyMat[num] = toggleGroupState; }
				perModelApplyMat[num] = EditorGUILayout.ToggleLeft(num + ": " + fileName, perModelApplyMat[num]);
				fileMatsDictionary[keys[num]] = (Material)EditorGUILayout.ObjectField("", fileMatsDictionary[keys[num]], typeof(Material), false); //create an object field that you can drag & drop materials into
				EditorGUILayout.EndHorizontal();
			}
			EditorGUILayout.EndToggleGroup();
		}
		EditorGUI.EndDisabledGroup();



		lastToggleGroupState = toggleGroupState;
		lastMaterialMode = materialMode;
	}*/

	private void ImportModels()
	{
		Undo.RegisterImporterUndo(importDirPath, "Import Models from " + importDirPath);
		string[] filesInDirectory = Directory.GetFiles(importDirPath);
		//RefreshModelsList();
		foreach (string filePath in filesInDirectory)
		{
			//these are all the 3D model file types Unity accepts
			if(filePath.EndsWith(".fbx") == true || filePath.EndsWith(".obj") == true || 
				filePath.EndsWith(".dae") == true || filePath.EndsWith(".3ds") == true ||
				filePath.EndsWith(".dxf") == true)
				{
				string relativeFilePath = GetRelativeFilePath(filePath);
				//Create Model Importer
				ModelImporter mi = (ModelImporter)AssetImporter.GetAtPath(relativeFilePath);

				#region Models Tab
				if (overwriteModelValues_Scene == true)
				{
					mi.globalScale = scaleFactor;
					mi.useFileScale = convertUnits;
					mi.bakeAxisConversion = bakeAxisConversion;
					mi.importBlendShapes = importBlendShapes;
					mi.importVisibility = importVisibility;
					mi.importCameras = importCameras;
					mi.importLights = importLights;
					mi.preserveHierarchy = preserveHierarchy;
					mi.sortHierarchyByName = sortHeiarchyByName;
				}

				if (overwriteModelValues_Meshes == true)
				{
					mi.meshCompression = meshCompression;
					mi.isReadable = readWriteEnabled; //this one may not work. I'm not sure
					mi.meshOptimizationFlags = optimizeMesh;
					mi.addCollider = generateColliders;
				}

				if (overwriteModelValues_Geometry == true)
				{
					mi.keepQuads = keepQuads;
					mi.weldVertices = weldVertices;
					mi.indexFormat = indexFormat;
					//skipped legacy blend shape normals b/c I couldn't get it to work
					mi.importNormals = normals;
					//skipped blend shape normals b/c I couldn't get it to work.
					mi.normalCalculationMode = normalCalculationMode;
					mi.normalSmoothingSource = normalSmoothingSource;
					mi.normalSmoothingAngle = normalSmoothingAngle;
					mi.importTangents = tangentSpaceMode;
					mi.swapUVChannels = swapUVs;
					mi.generateSecondaryUV = generateLightmapUVs;
				}
				#endregion
				#region Rig Tab
				if (overwriteRigValues == true)
				{
					if(overwriteRigValues == true)
					{
						mi.animationType = animType;
						mi.skinWeights = skinWeights;
					}
				}
				#endregion
				#region Anim Tab
				if (overwriteAnimValues == true)
				{
					mi.importConstraints = importConstraints;
					mi.importAnimation = importAnimations;
				}
				#endregion
				/*#region Materials Tab
				if (overwriteMatValues == true)
				{
					mi.materialImportMode = ModelImporterMaterialImportMode.LegacyImport;


					
					
				}
				#endregion*/
				//=== Reimport this model with the new settings ===
				mi.SaveAndReimport();
			}
		}
	}
	string GetRelativeFilePath(string fullPath)
	{
		if (fullPath == string.Empty)
		{
			return "";
		}

		string relativeFilePath = "Assets" + fullPath.Substring(Application.dataPath.Length);
		return relativeFilePath;
	}
	/*void RefreshModelsList()
	{
		//Undo.RegisterImporterUndo(importDirPath, "Refresh Models from " + importDirPath);
		perModelApplyMat.Clear();

		Dictionary<string, Material> newDictionary = new Dictionary<string, Material>();

		//NOTE: Line below says "Directory", not "Dictionary"
		string[] filesInFolder = Directory.GetFiles(importDirPath); //get all the files in the folder

		foreach (string filePath in filesInFolder)
		{
			if (filePath.EndsWith(".fbx") == true || filePath.EndsWith(".obj") == true ||
				filePath.EndsWith(".dae") == true || filePath.EndsWith(".3ds") == true ||
				filePath.EndsWith(".dxf") == true) //for each file that is a model file
			{
				string relativePath = GetRelativeFilePath(filePath); //get its relative path
				Material tempMat = null; //create an empty temp material and initialize it to null
				if(fileMatsDictionary.ContainsKey(relativePath) == true) //if the existing dictionary of materials already contains an entry with the same path
				{
					tempMat = fileMatsDictionary[relativePath]; //get the material at that path and store it in the temp material
				}
				newDictionary.Add(relativePath, tempMat); //add a new entry to the new dictionary that contains the files relative path as the key and the accociated material, if any, as the Value.
				perModelApplyMat.Add(true);
			}
		}

		fileMatsDictionary = newDictionary;
	}*/
}
